UNIT Objects;

{$I define.pp}

interface

CONST
{$IFDEF BIT_16}                                       { 16 BIT DEFINITION }
   MaxBytes = 65520;                                  { Maximum data size }
{$ELSE}                                               { 32 BIT DEFINITION }
   MaxBytes = 128*1024*1024;                          { Maximum data size }
{$ENDIF}
   MaxWords = MaxBytes DIV SizeOf(Word);              { Max word data size }
   MaxPtrs = MaxBytes DIV SizeOf(Pointer);            { Max ptr data size }
   MaxCollectionSize = MaxBytes DIV SizeOf(Pointer);  { Max collection size }

TYPE
   TCharSet = set of Char;                            { Character set }
   PCharSet = ^TCharSet;                              { Character set ptr }
   PString = ^String;                                 { String pointer }
   FNameStr = String;                                 { Linux filename }

   WordRec = record Lo, Hi: Byte end;                 { Word to bytes }
   LongRec = record Lo, Hi: Word end;                 { LongInt to words }

   TByteArray = ARRAY [0..MaxBytes-1] Of Byte;        { Byte array }
   PByteArray = ^TByteArray;                          { Byte array pointer }

   TWordArray = ARRAY [0..MaxWords-1] Of Word;        { Word array }
   PWordArray = ^TWordArray;                          { Word array pointer }

   TPointerArray = Array [0..MaxPtrs-1] Of Pointer;   { Pointer array }
   PPointerArray = ^TPointerArray;                    { Pointer array ptr }

   TPoint = record X,Y:Integer end;

   TRect = OBJECT
      A, B: TPoint;                                   { Corner points }
      FUNCTION Empty: Boolean;
      FUNCTION Equals (R: TRect): Boolean;
      FUNCTION Contains (P: TPoint): Boolean;
      PROCEDURE Copy (R: TRect);
      PROCEDURE Union (R: TRect);
      PROCEDURE Intersect (R: TRect);
      PROCEDURE Move (ADX, ADY: Integer);
      PROCEDURE Grow (ADX, ADY: Integer);
      PROCEDURE Assign (XA, YA, XB, YB: Integer);
   END;
   TObject = OBJECT
      CONSTRUCTOR Init;
      PROCEDURE Free;
      DESTRUCTOR Done; Virtual;
   END;
   PObject = ^TObject;

FUNCTION NewStr (Const S: String): PString;
PROCEDURE DisposeStr (P: PString);
PROCEDURE Abstract;
FUNCTION LongMul (X, Y: Integer): LongInt;
FUNCTION LongDiv (X: Longint; Y: Integer): Integer;

{ ---------- TPoint functions ------------ }

procedure TPointInc(var A:TPoint; B:TPoint);
procedure TPointDec(var A:TPoint; B:TPoint);

implementation

PROCEDURE CheckEmpty (Var Rect: TRect);
 BEGIN With Rect Do Begin
  If (A.X >= B.X) OR (A.Y >= B.Y) Then Begin      { Zero or reversed }
   A.X := 0;                                      { Clear a.x }
   A.Y := 0;                                      { Clear a.y }
   B.X := 0;                                      { Clear b.x }
   B.Y := 0;                                      { Clear b.y }
  End;
 End END;

FUNCTION TRect.Empty: Boolean;
 BEGIN
   Empty := (A.X >= B.X) OR (A.Y >= B.Y);             { Empty result }
 END;

FUNCTION TRect.Equals (R: TRect): Boolean;
 BEGIN
   Equals := (A.X = R.A.X) AND (A.Y = R.A.Y) AND
   (B.X = R.B.X) AND (B.Y = R.B.Y);                   { Equals result }
 END;

FUNCTION TRect.Contains (P: TPoint): Boolean;
 BEGIN
   Contains := (P.X >= A.X) AND (P.X < B.X) AND
     (P.Y >= A.Y) AND (P.Y < B.Y);                    { Contains result }
 END;

PROCEDURE TRect.Copy (R: TRect);
 BEGIN
   A := R.A;                                          { Copy point a }
   B := R.B;                                          { Copy point b }
 END;

PROCEDURE TRect.Union (R: TRect);
 BEGIN
   If (R.A.X < A.X) Then A.X := R.A.X;                { Take if smaller }
   If (R.A.Y < A.Y) Then A.Y := R.A.Y;                { Take if smaller }
   If (R.B.X > B.X) Then B.X := R.B.X;                { Take if larger }
   If (R.B.Y > B.Y) Then B.Y := R.B.Y;                { Take if larger }
 END;

PROCEDURE TRect.Intersect (R: TRect);
 BEGIN
   If (R.A.X > A.X) Then A.X := R.A.X;                { Take if larger }
   If (R.A.Y > A.Y) Then A.Y := R.A.Y;                { Take if larger }
   If (R.B.X < B.X) Then B.X := R.B.X;                { Take if smaller }
   If (R.B.Y < B.Y) Then B.Y := R.B.Y;                { Take if smaller }
   CheckEmpty(Self);                                  { Check if empty }
 END;

PROCEDURE TRect.Move (ADX, ADY: Integer);
 BEGIN
   Inc(A.X, ADX);                                     { Adjust A.X }
   Inc(A.Y, ADY);                                     { Adjust A.Y }
   Inc(B.X, ADX);                                     { Adjust B.X }
   Inc(B.Y, ADY);                                     { Adjust B.Y }
 END;

PROCEDURE TRect.Grow (ADX, ADY: Integer);
 BEGIN
   Dec(A.X, ADX);                                     { Adjust A.X }
   Dec(A.Y, ADY);                                     { Adjust A.Y }
   Inc(B.X, ADX);                                     { Adjust B.X }
   Inc(B.Y, ADY);                                     { Adjust B.Y }
   CheckEmpty(Self);                                  { Check if empty }
 END;

PROCEDURE TRect.Assign (XA, YA, XB, YB: Integer);
 BEGIN
   A.X := XA;                                         { Hold A.X value }
   A.Y := YA;                                         { Hold A.Y value }
   B.X := XB;                                         { Hold B.X value }
   B.Y := YB;                                         { Hold B.Y value }
 END;

TYPE
   DummyObject = OBJECT (TObject)                     { Internal object }
     Data: RECORD END;                                { Helps size VMT link }
   END;

CONSTRUCTOR TObject.Init;
 VAR LinkSize: LongInt; Dummy: DummyObject;
 BEGIN
   LinkSize := LongInt(@Dummy.Data)-LongInt(@Dummy);  { Calc VMT link size }
   FillChar(Pointer(LongInt(@Self)+LinkSize)^,
     SizeOf(Self)-LinkSize, #0);                      { Clear data fields }
 END;

PROCEDURE TObject.Free;
 BEGIN
   Dispose(PObject(@Self), Done);                     { Dispose of self }
 END;

DESTRUCTOR TObject.Done;
 BEGIN END;                                           { Abstract method }

FUNCTION NewStr (Const S: String): PString;
 VAR P: PString;
 BEGIN
   If (S = '') Then P := Nil Else Begin               { Return nil }
     GetMem(P, Length(S) + 1);                        { Allocate memory }
     If (P<>Nil) Then P^ := S;                        { Hold string }
   End;
   NewStr := P;                                       { Return result }
 END;

PROCEDURE DisposeStr (P: PString);
 BEGIN
   If (P <> Nil) Then FreeMem(P, Length(P^) + 1);     { Release memory }
 END;

PROCEDURE Abstract;
 BEGIN
  writeln('Abstract method call ...');
  Halt;                                               { Abstract error }
 END;

FUNCTION LongMul (X, Y: Integer): LongInt;
 BEGIN
 {$IFDEF ASM_FPK}
  ASM
   MOVZWL X, %EAX;                                    { Fetch x value }
   MOVZWL Y, %EDX;                                    { Fetch y value }
   IMULL %EDX, %EAX;                                  { Multiply results }
   MOVL %EAX, __RESULT;                               { Return result }
  END;
 {$ENDIF}
 END;

FUNCTION LongDiv (X: LongInt; Y: Integer): Integer;
 BEGIN
 {$IFDEF ASM_FPK}
  ASM
   MOVL X, %EAX;                                      { Fetch x value }
   MOVZWL Y, %ECX;                                    { Fetch y value }
   IDIV %ECX;                                         { Divid y into x }
   MOVL %EAX, __RESULT;                               { Return result }
  END;
 {$ENDIF}
 END;

{ ---------- TPoint functions ------------ }

procedure TPointInc(var A:TPoint; B:TPoint);
 begin inc(A.X,B.X); inc(A.Y,B.Y) end;
procedure TPointDec(var A:TPoint; B:TPoint);
 begin dec(A.X,B.X); dec(A.Y,B.Y) end;

end.
